@@ -55,22 +55,15 @@ from .. import __version__
     ),
     type=int,
 )
-@click.option(
-    "--mode",
-    "-d",
-    default="median",
-    show_default=True,
-    help=(
-        "Statistical method to use for detection. 'median' for median"
-        " background filter, stat for t-test."
-    ),
-)
 @click.option(
     "--pearson",
     "-p",
     default=None,
     show_default=True,
-    help=("Threshold to apply when detecting pattern changes."),
+    help=(
+        "Threshold to apply when detecting pattern changes. "
+        "A default value is selected based on the kernel."
+    ),
     type=float,
 )
 @click.option(
@@ -79,24 +72,48 @@ from .. import __version__
     default=0.10,
     show_default=True,
     help=(
-        "Minimum proportion of nonzero pixels to consider a region. "
+        "Minimum proportion of nonzero pixels required to consider a region. "
         "Smaller values allows lower coverage regions, but "
         "increase false positives."
     ),
     type=float,
 )
+@click.option(
+    "--cnr",
+    "-c",
+    default=1.0,
+    show_default=True,
+    help=(
+        "Contrast-to-noise-ratio threshold used to filter out positions with "
+        "high technical variations relative to biological variations."
+    ),
+    type=float,
+)
 @click.option(
     "--no-subsample",
-    "-s",
+    "-S",
     is_flag=True,
     help="Disable subsampling of input matrices to the same coverage.",
 )
+@click.option(
+    "--no-filter",
+    "-F",
+    is_flag=True,
+    help=(
+        "Completely disable pearson, cnr and density filtering. Mostly "
+        "for debugging. All input positions are returned, but results "
+        "will be noisy."
+    ),
+)
 @click.option(
     "--n-cpus",
     "-n",
     default=1,
     show_default=False,
-    help="Number of CPUs to use for parallel tasks",
+    help=(
+        "Number of CPUs to use for parallel tasks. It is recommended to set "
+        "at most to the number of input samples."
+    ),
 )
 @click.argument("cool_files", type=str)
 @click.argument("conditions", type=str)
@@ -111,12 +128,17 @@ def pareidolia_cmd(
     region,
     max_dist,
     no_subsample,
+    no_filter,
     pearson,
     density,
-    mode,
+    cnr,
     n_cpus,
 ):
-    """Run the pattern change detection pipeline"""
+    """Run the pattern change detection pipeline. Given a list of cool files
+    and associated conditions, compute pattern intensity change from one condition
+    relative to the control. The first condition occuring in the list is the control.
+    For all patterns that pass the quality filters, a differential score
+    (condition - control) and a contrast-to-noise ratio are returned."""
 
     # Attempt to load chromosight kernel from the kernel name.
     try:
@@ -124,12 +146,16 @@ def pareidolia_cmd(
     # If the kernel name is not valid, assumes it's a custom kernel file
     except AttributeError:
         try:
-            _ = np.loadtxt(kernel)
+            kernel = np.loadtxt(kernel)
         except OSError:
             raise ValueError(
                 "kernel must either be a valid kernel name or path to a"
                 " text file (see --help)."
             )
+    # Disable all filters if requested
+    if no_filter:
+        cnr = density = None
+        pearson = 0.0
     # Get lists from comma-separated items
     cool_files = _parse_cli_list(cool_files)
     conditions = _parse_cli_list(conditions)
@@ -145,8 +171,8 @@ def pareidolia_cmd(
         subsample=not no_subsample,
         pearson_thresh=pearson,
         density_thresh=density,
+        cnr_thresh=cnr,
         n_cpus=n_cpus,
-        mode=mode,
     )
 
     # Save results to text file
