@@ -151,7 +151,9 @@ def get_win_density(
         density[frame.row, frame.col].A1 * win_area / frame.data
     )
     # Trim the frame out from the signal
-    density = density[win_size - 1 : -win_size + 1, win_size - 1 : -win_size + 1]
+    density = density[
+        win_size - 1 : -win_size + 1, win_size - 1 : -win_size + 1
+    ]
     if sym_upper:
         density = sp.triu(density)
     return density
