@@ -39,7 +39,9 @@ def get_min_contacts(
         if region is None:
             contacts[i] = clr.info["sum"]
         else:
-            contacts[i] = clr.matrix(balance=False, sparse=True).fetch(region).sum()
+            contacts[i] = (
+                clr.matrix(balance=False, sparse=True).fetch(region).sum()
+            )
     # Return the minimum coverage value (in number of contacts)
     return int(min(contacts))
 
@@ -156,7 +158,9 @@ def make_density_filter(
     # Filter out regions where contacts were too sparse in all contact matrices
     # Pixel density is first converted to a boolean matrix (pass / fail)
     filters = map(
-        lambda m: pad.get_win_density(m, win_size=win_size, sym_upper=sym_upper)
+        lambda m: pad.get_win_density(
+            m, win_size=win_size, sym_upper=sym_upper
+        )
         > density_thresh,
         mats,
     )
@@ -166,16 +170,20 @@ def make_density_filter(
     return inter_filter
 
 
-def _ttest_matrix(samples: pd.DataFrame, control: str) -> Tuple[sp.csr_matrix, float]:
+def _ttest_matrix(
+    samples: pd.DataFrame, control: str
+) -> Tuple[sp.csr_matrix, float]:
     """
     Performs pixel-wise t-test comparisons between conditions to detect differential
     interactions.
     """
     # Compute background for each condition
-    arr_control = np.dstack([m.data for m in samples.mat[samples.cond == control]])[
-        0, :, :
-    ]
-    arr_alt = np.dstack([m.data for m in samples.mat[samples.cond != control]])[0, :, :]
+    arr_control = np.dstack(
+        [m.data for m in samples.mat[samples.cond == control]]
+    )[0, :, :]
+    arr_alt = np.dstack(
+        [m.data for m in samples.mat[samples.cond != control]]
+    )[0, :, :]
     diff = samples["mat"][0]
     diff.data = pas.vec_ttest(arr_control, arr_alt)
     # The threshold is the t-value corresponding to p=0.05
@@ -187,11 +195,15 @@ def _ttest_matrix(samples: pd.DataFrame, control: str) -> Tuple[sp.csr_matrix, f
 
 
 def _median_bg_subtraction(
-    samples: pd.DataFrame, control: str
-) -> Tuple[sp.csr_matrix, float]:
+    samples: pd.DataFrame,
+    control: str,
+    cnr_thresh: Optional[float] = 1.0,
+    cnr_max: float = 10.0,
+) -> Tuple[sp.csr_matrix, sp.csr_matrix]:
     """
     Performs the median background subtraction to extract differential signal
-    from multiple Hi-C matrix.
+    from multiple Hi-C matrix. Returns the filtered differential matrix and
+    the contrast-to-noise-ratio matrix used for the filtering.
     """
     # Compute background for each condition
     backgrounds = samples.groupby("cond")["mat"].apply(
@@ -202,23 +214,29 @@ def _median_bg_subtraction(
         lambda g: pad.get_sse_mat(g.reset_index(drop=True))
     )
     conditions = np.unique(samples.cond)
-    # Compute difference between conditions and signal
+    # Compute difference between conditions and contrast
     # to noise ratio
-    snr = np.zeros(sse[0].data.shape)
+    cnr = sse[0].copy()
+    cnr.data = np.zeros(sse[0].data.shape)
     diff = sp.csr_matrix(sse[0].shape)
     for c in conditions:
-        snr += backgrounds[c].data / np.sqrt(sse[c].data)
+        if cnr_thresh is not None:
+            cnr.data += backgrounds[c].data / np.sqrt(sse[c].data)
         if c != control:
             # Break ties to preserve sparsity (do not introduce 0s)
             ties = backgrounds[c].data == backgrounds[control].data
-            backgrounds[c].data[ties] += 1e-06
+            backgrounds[c].data[ties] += 1e-08
             diff += backgrounds[c] - backgrounds[control]
-    snr /= len(conditions)
+    cnr.data /= len(conditions)
+    # Erase spurious or extreme values
+    cnr.data[cnr.data < 0.0] = 0.0
+    cnr.data[cnr.data > cnr_max] = cnr_max
     # Use average difference to first background as change metric
     diff.data /= len(conditions) - 1
     # Threshold data on background / sse value
-    diff.data[snr < 1.0] = 0.0
-    return diff
+    if cnr_thresh is not None:
+        diff.data[cnr.data < cnr_thresh] = 0.0
+    return diff, cnr
 
 
 def detection_matrix(
@@ -229,15 +247,21 @@ def detection_matrix(
     max_dist: Optional[int] = None,
     pearson_thresh: Optional[float] = None,
     density_thresh: Optional[float] = None,
-    mode="median",
+    cnr_thresh: Optional[float] = 1.0,
     n_cpus: int = 4,
-) -> Tuple[Optional[sp.csr_matrix], Optional[float]]:
+) -> Tuple[Optional[sp.csr_matrix], Optional[sp.csr_matrix]]:
     """
     Run the detection process for a single chromosome or region. This is abstracted from all
     notions of chromosomes and genomic coordinates.
     """
     # We consider the matrix is symmetric upper (i.e. intrachromosomal)
     sym_upper = True
+    # Diagonals will be trimmed at max_dist with a margin for convolution
+    if max_dist is None:
+        trim_dist = None
+    else:
+        mat_size = samples.cool[0].matrix(sparse=True).fetch(region).shape[0]
+        trim_dist = min(mat_size, max_dist + max(kernel.shape))
     # Compute number of contacts in the matrix with the lowest coverage
     if subsample:
         min_contacts = get_min_contacts(samples.cool, region=region)
@@ -261,24 +285,25 @@ def detection_matrix(
     print(f"{region} preprocessed", file=sys.stderr)
     # Return nothing if the matrix is smaller than kernel
     if np.any(np.array(samples["mat"][0].shape) <= np.array(kernel.shape)):
-        return None
+        return None, None
     # Retrieve the indices of bins which are valid in all samples (not missing
     # because of repeated sequences or low coverage)
     common_bins = pap.get_common_valid_bins(samples["mat"])
     # Trim diagonals beyond max_dist (with kernel margin for the convolution)
     # to spare resources
-    samples["mat"] = map_fun(
-        cup.diag_trim, zip(samples["mat"], it.repeat(max_dist + max(kernel.shape)))
-    )
+    if trim_dist is not None:
+        samples["mat"] = map_fun(
+            cup.diag_trim, zip(samples["mat"], it.repeat(trim_dist))
+        )
     # Generate a missing mask from these bins
     missing_mask = cup.make_missing_mask(
         samples["mat"][0].shape,
         common_bins,
         common_bins,
-        max_dist=max_dist + max(kernel.shape),
+        max_dist=trim_dist,
         sym_upper=sym_upper,
     )
-    # Remove all missing values form each sample's matrix
+    # Remove all missing values from each sample's matrix
     samples["mat"] = map_fun(
         cup.erase_missing,
         zip(
@@ -293,7 +318,7 @@ def detection_matrix(
     # Compute a density filter: regions with sufficient proportion of nonzero
     # pixels in kernel windows, in all samples. We will use it for downstream
     # which filter
-    if density_thresh is not None:
+    if (density_thresh is not None) and (density_thresh > 0):
         density_filter = make_density_filter(
             samples["mat"],
             density_thresh=density_thresh,
@@ -321,11 +346,14 @@ def detection_matrix(
     # Get the union of nonzero coordinates across all samples
     total_nnz_set = pap.get_nnz_union(samples["mat"])
     # Fill zeros at these coordinates
-    samples["mat"] = samples["mat"].apply(lambda cor: pap.fill_nnz(cor, total_nnz_set))
-
+    samples["mat"] = samples["mat"].apply(
+        lambda cor: pap.fill_nnz(cor, total_nnz_set)
+    )
     # Erase pixels where all samples are below pearson threshold
     if pearson_thresh is not None:
-        pearson_fail = [(m.data < pearson_thresh).astype(bool) for m in samples["mat"]]
+        pearson_fail = [
+            (m.data < pearson_thresh).astype(bool) for m in samples["mat"]
+        ]
         pearson_fail = np.bitwise_and.reduce(pearson_fail)
         # Threshold maps using pearson correlations to reduce noisy detections
         for i, m in enumerate(samples["mat"]):
@@ -336,20 +364,15 @@ def detection_matrix(
         pool.close()
 
     # Use median background
-    if mode == "median":
-        diff = _median_bg_subtraction(samples, control)
-    # Use t-test on each pixel (untested, probably doesn't work well)
-    else:
-        diff = _ttest_matrix(samples, control)
+    diff, cnr = _median_bg_subtraction(samples, control, cnr_thresh)
 
     # Erase pixels which do not pass the density filter in all samples
-    if density_thresh is not None:
+    if (density_thresh is not None) and (density_thresh > 0):
         diff = diff.multiply(density_filter)
     # Remove all values beyond user-specified max_dist
     if max_dist is not None:
         diff = cup.diag_trim(diff, max_dist + 2)
-
-    return diff
+    return diff, cnr
 
 
 def change_detection_pipeline(
@@ -362,9 +385,9 @@ def change_detection_pipeline(
     min_dist: Optional[int] = None,
     subsample: bool = True,
     pearson_thresh: Optional[float] = None,
-    density_thresh: float = 0.10,
+    density_thresh: Optional[float] = 0.10,
+    cnr_thresh: Optional[float] = 1.0,
     n_cpus: int = 4,
-    mode="median",
 ) -> pd.DataFrame:
     """
     Run end to end pattern change detection pipeline on input cool files. A
@@ -376,53 +399,47 @@ def change_detection_pipeline(
     matrix can be supplied directly instead. maximum scanning distance can be
     specified directly (in basepairs) to override the kernel default value.
 
-    Positions with significant changes will be reported in a pandas
-    dataframe. Significance is determined based on the percentile threshold,
-    between 1 and 100. Optionally, a 2D bed file with positions of interest can
-    be specified, in which case change value at these positions will be
-    reported instead. When using a bed2d file, the threshold is optional (one
-    can report either scores at all positions, or only where they are
-    significant).
-
     Positive diff_scores mean the pattern intensity was increased relative to
     control (first condition).
 
+    Positions with significant changes will be reported in a pandas
+    dataframe. In addition to the score, a contrast-to-noise ratio between 0 and
+    10 is given to give an estimation of the signal quality. Optionally, a 2D
+    bed file with positions of interest can be specified, in which case change
+    value at these positions will be reported instead. When using a bed2d file.
+
     Parameters
     ----------
-    cool_files : Iterable of strs
+    cool_files :
         The list of paths to cool files for the input samples.
-    conditions : Iterable of strs
+    conditions :
         The list of conditions matching the samples.
-    kernel : np.array of floats or str
+    kernel :
         Either the kernel to use as pattern as a numpy array, or the name of a
         valid chromosight pattern.
-    bed2d_file : str or None
+    bed2d_file :
         Path to a bed2D file containing a list of 2D positions. If this is
         provided, pattern changes at these coordinates will be quantified.
         Otherwise, they will be detected based on a threshold.
-    region : str or iterable of strs
+    region :
         Either a single UCSC format region string, or a list of multiple
         regions. The analysis will be restricted to those regions.
-    max_dist : int or None
+    max_dist :
         Maximum interaction distance (in basepairs) to consider in the analysis.
         If this is not specified and a chromosight kernel was specified, the
         default max_dist for that kernel is used. If the case of a custom kernel,
         the whole matrix will be scanned if no max_dist is specified.
-    subsample : bool
+    subsample :
         Whether all input matrices should be subsampled to the same number of
         contacts as the least covered sample.
-    pearson_thresh: float or None
+    pearson_thresh :
         The pearson correlation threshold to use when detecting patterns. If None,
         the default value for the kernel is used.
-    density_thresh: float
+    density_thresh :
         The pixel density threshold to require. Low coverage windows with a
         proportion of nonzero pixels below this value are discarded.
-    n_cpus : int
+    n_cpus :
         Number of CPU cores to allocate for parallel operations.
-    mode : str
-        Algorithm used for change detection. Can be median or stat. Median is
-        the default, stat is experimental and should only be used with at least
-        3 samples per condition.
 
     Returns
     -------
@@ -435,12 +452,6 @@ def change_detection_pipeline(
             "The lists of cool files and conditions must have the same length"
         )
 
-    if mode != "median":
-        print(
-            "Running in t-test mode, t-values will be used as diff scores.",
-            file=sys.stderr,
-        )
-
     # If a pattern name was provided, load corresponding chromosight kernel
     if isinstance(kernel, str):
         kernel_name = kernel
@@ -465,13 +476,19 @@ def change_detection_pipeline(
             "Kernel must either be a valid chromosight pattern name, or a 2D numpy.ndarray of floats"
         )
     # Associate samples with their conditions
-    samples = pd.DataFrame({"cond": conditions, "cool": pai.get_coolers(cool_files)})
-    print(f"Changes will be computed relative to condition: {samples.cond.values[0]}")
+    samples = pd.DataFrame(
+        {"cond": conditions, "cool": pai.get_coolers(cool_files)}
+    )
+    print(
+        f"Changes will be computed relative to condition: {samples.cond.values[0]}"
+    )
     # Define each chromosome as a region, if None specified
     clr = samples.cool.values[0]
     if max_dist is not None:
         max_dist = max_dist // clr.binsize
-    if min_dist is not None:
+    if min_dist is None:
+        min_dist = 0
+    else:
         min_dist = min_dist // clr.binsize
     if region is None:
         regions = clr.chroms()[:]["name"].tolist()
@@ -489,17 +506,18 @@ def change_detection_pipeline(
         "bin1",
         "bin2",
         "diff_score",
+        "cnr",
     ]
     if bed2d_file:
         positions = cio.load_bed2d(bed2d_file)
-        for col in ["diff_score", "bin1", "bin2"]:
+        for col in ["diff_score", "cnr", "bin1", "bin2"]:
             positions[col] = np.nan
     else:
         positions = pd.DataFrame(columns=pos_cols)
     for reg in regions:
         # Subset bins to the range of interest
         bins = clr.bins().fetch(reg).reset_index(drop=True)
-        diff = detection_matrix(
+        diff, cnr = detection_matrix(
             samples,
             kernel,
             region=reg,
@@ -508,7 +526,7 @@ def change_detection_pipeline(
             pearson_thresh=pearson_thresh,
             density_thresh=density_thresh,
             n_cpus=n_cpus,
-            mode=mode,
+            cnr_thresh=cnr_thresh,
         )
 
         # If the matrix was too small or no difference was found, skip it
@@ -517,7 +535,9 @@ def change_detection_pipeline(
         # If positions were provided, return the change value for each of them
         if bed2d_file:
             tmp_chr = reg.split(":")[0]
-            tmp_rows = (positions.chrom1 == tmp_chr) & (positions.chrom2 == tmp_chr)
+            tmp_rows = (positions.chrom1 == tmp_chr) & (
+                positions.chrom2 == tmp_chr
+            )
             # If there are no positions of interest on this chromosome, just
             # skip it
             if not np.any(tmp_rows):
@@ -526,7 +546,9 @@ def change_detection_pipeline(
             # Convert both coordinates from genomic coords to bins
             for i in [1, 2]:
                 tmp_pos["chrom"] = tmp_pos[f"chrom{i}"]
-                tmp_pos["pos"] = (tmp_pos[f"start{i}"] + tmp_pos[f"end{i}"]) // 2
+                tmp_pos["pos"] = (
+                    tmp_pos[f"start{i}"] + tmp_pos[f"end{i}"]
+                ) // 2
                 tmp_pos[f"bin{i}"] = coords_to_bins(clr, tmp_pos).astype(int)
                 # Save bin coordinates from current chromosome to the full table
                 positions.loc[tmp_rows, f"bin{i}"] = tmp_pos[f"bin{i}"]
@@ -535,6 +557,9 @@ def change_detection_pipeline(
             positions.loc[tmp_rows, "diff_score"] = diff[
                 tmp_pos.start1 // clr.binsize, tmp_pos.start2 // clr.binsize
             ].A1
+            positions.loc[tmp_rows, "cnr"] = cnr[
+                tmp_pos.start1 // clr.binsize, tmp_pos.start2 // clr.binsize
+            ].A1
         # Otherwise report individual spots of change using chromosight
         else:
             # Pick "foci" of changed pixels and their local maxima
@@ -561,6 +586,7 @@ def change_detection_pipeline(
             # No position found, go to next region
             except AttributeError:
                 continue
+            tmp_pos["cnr"] = cnr[tmp_pos.bin1, tmp_pos.bin2].A1
             # Append new chromosome's rows
             positions = pd.concat([positions, tmp_pos], axis=0)
             # For 1D patterns (e.g. borders) set diagonal positions.
@@ -572,6 +598,5 @@ def change_detection_pipeline(
     positions = positions.loc[
         abs(positions.bin2 - positions.bin1) >= min_dist, :
     ].reset_index(drop=True)
-    print(min_dist)
     print(positions)
     return positions
